#ifndef IOfileServer_h
#define IOfileServer_h

#include <vector>
#include <string>

#include "Event/serviceI.h"
#include "Event/serverVI.h"

#include "Event/IOBase.h"
#include "Event/IOfileVI.h"

//----------------------------------------------
//! protected server that operates the input and output persistent data files
/*!
\e IOfileServer should be a base class of the dataIOPersistentServer class.
In our case Persistent is ROOT, and the class is named dataIOROOTServer.<br>
\e Users can define the input/output file names  of the following 
system defined IOfiles "input", "output" and "histo".
<ul>
	<li> Users can add files into the server with the \c defioneIOfiles() method.
	<li> The IO file operations are defined in IOfileVI.
	<li> The IOfileServer has a serverVI with the input/output data files IOfileVI.
	There are three names already used by the system: 
	<ul>
		<li> input persistent data file named \e "input"
		<li> output persistent data file named \e "output"
		<li> output histogram file named "\e histo"
	</ul>
	<li> The user can set the fine names of the "input", "output", "histo" IOfiles,
	via the services of serviceI. see \c defineOption() implementation.
	<li> The IOfileServer uses it bare class IOfileVI to define a virtual file
	that is alive during the RUN and keeps track of the number of events processed,
	opens and closes files, etc.
</ul>
*/
//----------------------------------------------
//  J.A Hernando, Santa Cruz, CA 06/15/00
//----------------------------------------------

//####################################
class IOfileServer : public IOfileVI, public serviceI
//####################################
{
public:

	//! open the files in server named "input", "output" and "histo"
	virtual void open();
	//! close the files in server named "input", "output" and "histo"
	virtual void close();

	//! virtual method to know if there is one event more in the file
	virtual bool nextEvent();
	//! virtual method to read the next event - overwrite by the derived class
	virtual void readEvent();
	//! virtual method to write an event - overwrite by the derived class
	virtual void writeEvent();
	//! virtual method to skip next event - overwrite by the derived class
	virtual void skipEvent();

	//! virtual method to write Out information of this class
	virtual void writeOut() const;

protected:

	//! constructor - execute the pure virtual method \c defineIOfiles()
	IOfileServer(); 
	//! destructor
	virtual ~IOfileServer();   

	//! the User should declare the IOfiles in this method
	virtual void defineIOfiles() {};

	//! returns true is there is a file with that name in server
	bool search(std::string name) const {return m_server->search(name);}

	//! add a file into the server
	virtual void addIOfile(std::string name, IOfileVI* file) {m_server->add(name,file);}
	
	//! returns the IOfileVI file in server
	IOfileVI* getIOfile(std::string name) const;

protected:

	//! returns the next input file name to open
	std::string getNextInFileName();
	//! returns the histogram file name
	std::string getHistoFileName() const {return m_histoFile;}
	//! returns the output file name
	std::string getOutFileName()   const {return m_OutFile;};

	//! define the options optionVI
	virtual void defineOption();
	//! method to define the names of the input/output IO files
	virtual void setOption(std::string par, std::string v); // overwriten

private:

	//! display message
	bool displayFrequency() const;

private:
	
	serverVI<IOfileVI>* m_server;

	//! pointer to avoid the search
	IOfileVI* m_input;
	IOfileVI* m_output;
	IOfileVI* m_histo;

	//! number of the input file currently open
	int m_ifile;
	//! List of input files
	std::vector<std::string> m_InFileList;
	//! name of the last input input file
	std::string m_InFile;
	//! name of the histo file
	std::string m_histoFile;
	//! name of the output file
	std::string m_OutFile;

	//! frequency to display a message
	int m_displayFreq;
};
#endif

